/*
 *	PostgreSQL type definitions for IP addresses (both v4 and v6)
 *
 *	$Id: ip_address.c,v 1.2 2002/09/18 13:53:24 bortzmeyer Exp $
 */

#include "generic_addr.h"
#include "postgres.h"

/*
 *	Various forward declarations:
 */

generic_addr *ip_address_in (char *str);
char *ip_address_out (generic_addr * addr);

bool ip_address_lt (generic_addr * l, generic_addr * right);
bool ip_address_le (generic_addr * l, generic_addr * right);
bool ip_address_eq (generic_addr * l, generic_addr * right);
bool ip_address_ge (generic_addr * l, generic_addr * right);
bool ip_address_gt (generic_addr * l, generic_addr * right);

bool ip_address_ne (generic_addr * l, generic_addr * right);

int4 ip_address_cmp (generic_addr * l, generic_addr * right);

/*
 *	IP_ADDRESS reader.
 */

generic_addr *
ip_address_in (char *str)
{
  generic_addr *result, val;
  result = MALLOC (sizeof (generic_addr));
  INIT (*result);
  val = text_to_address (str);
  if (IS_UNSPEC (val))
    {
      elog (ERROR, "ip_address_in: invalid IP_ADDRESS \"%s\"", str);
      return (NULL);
    }
  memcpy (result, &val, sizeof(generic_addr));
  return (result);
}

/*
 *	IP_ADDRESS output function.
 */

char *
ip_address_out (generic_addr * num)
{
  char *result, *val;

  if (num == NULL)
    return (NULL);
  result = (char *) MALLOC (INET6_ADDRSTRLEN);
  val = address_to_text (*num);
  strcpy (result, val); 
  /* elog (NOTICE, "still in ip_address_out (getting %s)", result);  */
  return (result);
}

/*
 *	Boolean tests for magnitude.
 */

bool
ip_address_lt (generic_addr * l, generic_addr * r)
{
  return (is_lt (*l, *r));
};

bool
ip_address_le (generic_addr * l, generic_addr * r)
{
  return (is_le (*l, *r));
};

bool
ip_address_eq (generic_addr * l, generic_addr * r)
{
  return (are_equal (*l, *r));
};

bool
ip_address_ge (generic_addr * l, generic_addr * r)
{
  return (is_ge (*l, *r));
};

bool
ip_address_gt (generic_addr * l, generic_addr * r)
{
  return (is_gt (*l, *r));
};

bool
ip_address_ne (generic_addr * l, generic_addr * r)
{
  return (!are_equal (*l, *r));
};

/*
 *	Comparison function for sorting:
 */

int4
ip_address_cmp (generic_addr * l, generic_addr * r)
{
  return (cmp (*l, *r));
}

bool
is_ipv4 (generic_addr * a)
{
  return (IS_IPV4(*a));
}

bool
is_ipv6 (generic_addr * a)
{
  return (IS_IPV6(*a));
}

